@@ -379,6 +379,8 @@ url_get_method_name(URL this) {
       return "PUT";
     case DELETE:
       return "DELETE";
+    case OPTIONS:
+      return "OPTIONS";
     case HEAD:
      return "HEAD";
     case GET:
@@ -581,6 +583,14 @@ __url_parse(URL this, char *url)
   if (! post) {
     post = strstr(this->url, " PATCH");
   }
+	
+  if (! post) {
+    post = strstr(this->url, " OPTIONS");
+  }
+	
+  if (! post) {
+    post = strstr(this->url, " DELETE");
+  }
 
   if (post != NULL){
     if (!strncasecmp(post," PUT", 4)) {
@@ -591,6 +601,14 @@ __url_parse(URL this, char *url)
       this->method = POST;
       *post = '\0';
       post += 5;
+    } else if (!strncasecmp(post," DELETE", 7)) {
+      this->method = DELETE;
+      *post = '\0';
+      post += 7;
+    } else if (!strncasecmp(post," OPTIONS", 8)) {
+      this->method = OPTIONS;
+      *post = '\0';
+      post += 8;
     } else {
       this->method = PATCH;
       *post = '\0';
@@ -847,8 +865,22 @@ __url_set_hostname(URL this, char *str)
     memmove(str, str+n, len - n + 1);
   }
 
-  /* skip to end, slash, or port colon */
-  for (i = 0; str[i] && str[i] != '/' && str[i] != '#' && str[i] != ':'; i++);
+  /**
+   * Check for IPv6 address. The convention here is to use square brackets
+   * around the IPv6 address in order to have a clear delimitation between
+   * address and port
+   */
+  if (startswith("[", str)) {
+    /* skip to matching square bracket */
+    for (i = 0; str[i] && str[i] != ']'; i++);
+
+    if (str[i] == ']') {
+      i++;
+    }
+  } else {
+    /* skip to end, slash, or port colon */
+    for (i = 0; str[i] && str[i] != '/' && str[i] != '#' && str[i] != ':'; i++);
+  }
 
   this->hostname = xmalloc(i + 1);
   memset(this->hostname, '\0', i+1);
@@ -1162,7 +1194,8 @@ __url_escape(const char *s)
 {
   const char *p1;
   char *newstr, *p2;
-  int oldlen, newlen;
+  int oldlen, newlen, host_len;
+  char *path_start, *host_start;
 
   int encode_count = 0;
   int decode_count = 0;
@@ -1176,9 +1209,24 @@ __url_escape(const char *s)
     return (char *)s;
   }  
 
+  /* skip directly to path */
+  host_start = strstr(s, "//");
+  if (host_start) {
+    host_start += 2;
+  } else {
+    host_start = (char *)s;
+  }
+
+  path_start = strstr(host_start, "/");
+  if (path_start) {
+    path_start += 1;
+  } else { /* there is no path to escape */
+    return (char *)s;
+  }
+
   /* First, pass through the string to see if there's anything to do,
      and to calculate the new length.  */
-  for (p1 = s; *p1; p1++) {
+  for (p1 = path_start; *p1; p1++) {
     switch (decide_copy_method (p1)) {
       case CM_ENCODE:
         ++encode_count;
@@ -1195,13 +1243,16 @@ __url_escape(const char *s)
     return (char *)s; /* C const model sucks. */
 
   oldlen = p1 - s;
+  host_len = path_start - s;
   /* Each encoding adds two characters (hex digits), while each
      decoding removes two characters.  */
   newlen = oldlen + 2 * (encode_count - decode_count);
   newstr = xmalloc (newlen + 1);
 
-  p1 = s;
-  p2 = newstr;
+  /* copy unmodified to new_str up to path_start */
+  memcpy(newstr, s, host_len);
+  p1 = path_start;
+  p2 = newstr + host_len;
 
   while (*p1) {
     switch (decide_copy_method (p1)) {
